import abc
import threading
from abc import ABC
from dataclasses import dataclass
from datetime import datetime
from typing import Dict


@dataclass
class CacheEntry:
    key: str
    fetchIntervalInSeconds: int
    settings: Dict
    lastFetchTimestamp: float
    cachedData: Dict


lock = threading.Lock()


class MultiCacheKeyService(ABC):
    def __init__(self):
        self._cache = {}

    def get_data(self, cacheKey: str, fetchIntervalInSeconds: int, settings: Dict) -> Dict:
        with lock:
            if cacheKey not in self._cache:
                self._cache[cacheKey] = CacheEntry(cacheKey, fetchIntervalInSeconds, settings, 0, {})

            cacheEntry = self._cache[cacheKey]
            if self.__is_data_obsolete(cacheEntry):
                cacheEntry.cachedData = self._fetch_data(cacheEntry.settings)
            return cacheEntry.cachedData

    def __is_data_obsolete(self, cacheEntry: CacheEntry) -> bool:
        now = datetime.now().timestamp()
        if (now - cacheEntry.lastFetchTimestamp) > cacheEntry.fetchIntervalInSeconds:
            cacheEntry.lastFetchTimestamp = now
            return True

        return False

    @abc.abstractmethod
    def _fetch_data(self, settings: Dict) -> Dict:
        pass
